// Auto.java                                        MM 2014
/**
* Abstrakte Oberklasse fuer ein Auto mit Ausstattung.
*/
public abstract class Auto
{
  /**
  * Ausstattung des Autos:
  * Automatik, Klimaanlage, Navigationssystem,
  * Parksystem, Winterreifen.
  */
  private Ausstattung ausstattung;
  
  /**
  * Mietgrundpreis des Autos pro Tag in Euro.
  */
  private int grundPreis = 0;
  
  /**
  * Vierstellinge Endzahl fuer Kennzeichen,
  * wird mit jedem Kauf eines Wagens weitergezaehlt.
  */
  private static int nummer = 1000;
  
  /**
  * Neues Kennzeichen eines Autos,
  * setzt sich aus L-RA- und vierstellinger Endzahl
  * zusammen.
  */
  private String kennZeichen = "L-RA-";
  
/* ----------------------------------------------------- */

  /**
  * Konstruktor,
  * legt neues Kennzeichen fest,
  * setzt Ausstattung und Mietgrundpreis.
  * @param ausstattung Ausstattung des Autos
  * @param grundPreis Grundpreis pro Tag in Euro
  */
  public Auto( Ausstattung ausstattung, int grundPreis)
  {
    kennZeichen += nummer++;
    this.ausstattung = ausstattung;
    this.grundPreis = grundPreis;
  }
  
/* ----------------------------------------------------- */

  /**
  * Bestimmt den Wagentyp.
  * @return Wagentyp:
  *      "Kleinwagen" "Mittelklassewagen" "Oberklassewagen"
  */
  public String getTyp()
  {
    return getClass().getName();
  }
  
  /**
  * Liest Kennzeichen eines Autos.
  * @return L-RA-nnnn
  */
  public String getKennZeichen()
  {
    return kennZeichen;
  }
  
  /**
  * Reifen werden neu aufgezogen.
  * @param winter true,  falls es Winterreifen sind
  *               false, falls es Sommerreifen sind
  */
  public void reifenWechsel( boolean winter )
  {
    ausstattung.reifenWechsel( winter);
  }

  /**
  * Berechnet den Mietpreis in Abhaenigkeit vom Mietgrundpreis
  * und von dem Ausstattungsgesamtzuschlag.
  * @return Mietpreis pro Tag in Euro
  */
  public int berechneMietPreis()
  {
    return grundPreis + ausstattung.berechneGesamtZuschlag();
  }
  
  /**
  * Gibt als String zurueck:
  * Kennzeichen, Wagentyp,
  * vorhandene Ausstattung und Mietpreis.
  * @return Autobeschreibung
  */
  public String toString()
  {
    String str = "Kennzeichen: " + kennZeichen;
    str += "\nWagentyp: " + getTyp();
    str += "\nAusstattung: " + ausstattung;
    str += "\nTagespreis: " + berechneMietPreis() + "Euro";
    return str;
  }

/* ----------------------------------------------------- */

  /**
  * Testprogramm:
  * Erzeugt 6 Testautos, fuehrt einen Reifenwechsel durch
  * und gibt deren Autobeschreibung auf der Konsole aus.
  */
  public static void main( String[] args )
  {
    Ausstattung ausstattung;
    Auto auto;
    
    ausstattung
    = new Ausstattung( true, true, true, false, false);
    ausstattung.setSonst( "neu");
    auto =  new Kleinwagen( ausstattung, 50);
    System.out.println( "\nKleinwagen:\n" + auto);

    ausstattung
    = new Ausstattung( false, true, false, false, false);
    ausstattung.setSonst( "Multimedia"); 
    auto =  new Kleinwagen( ausstattung, 40);
    System.out.println( "\nKleinwagen:\n" + auto);

    ausstattung
    = new Ausstattung( false, true, true, true, false);
    auto = new Mittelklassewagen( ausstattung, 60);
    System.out.println( "\nMittelklassewagen:\n" + auto);

    ausstattung
    = new Ausstattung( true, true, true, false, false);
    auto = new Mittelklassewagen( ausstattung, 60);
    System.out.println( "\nMittelklassewagen:\n" + auto);

    ausstattung
    = new Ausstattung( false, true, true, false, false);
    ausstattung.setSonst( "Cabrio"); 
    auto = new Oberklassewagen( ausstattung, 90);
    System.out.println( "\nOberklassewagen:\n" + auto);

    ausstattung
    = new Ausstattung( true, true, true, true, false);
    auto = new Oberklassewagen( ausstattung, 80);
//    System.out.println( "\nOberklassewagen:\n" + auto);

    auto.reifenWechsel( true); 
    System.out.println
    ( "\nOberklassewagen nach Reifenwechsel:\n" + auto);
  }
}
  