// Ort.java                                         MM 2014
import java.util.*;                           // LinkedList

public class Ort
{
  /**
  * Liste aller Autos, die sich vor Ort befinden.
  */
  private LinkedList<Auto> alleAutos;
  
/* ----------------------------------------------------- */

  /**
  * Konstruktor, erzeugt leere Liste von Autos an.
  */
  public Ort()
  {
    alleAutos = new LinkedList<Auto>();
  }
  
/* ----------------------------------------------------- */

  /**
  * Gibt alle Autos vor Ort zurueck.
  * Falls keine vorhanden sind, ist die Liste leer.
  * @return Autos vor Ort
  */
  public LinkedList<Auto> getAlleAutos()
  {
    return alleAutos;
  }

  /**
  * Sucht alle Autos vor Ort mit gegebenen Wagentyp.
  * Falls keine vorhanden sind, ist die Liste leer.
  * @param typ Wagentyp:
  *      "Kleinwagen" "Mittelklassewagen" "Oberklassewagen"
  * @return Autos vom Wagentyp vor Ort
  */
  public LinkedList<Auto> getAlleAutos( String typ )
  {
    LinkedList<Auto> liste = new LinkedList<Auto>();
    for( Auto auto: alleAutos)
      if( auto.getTyp().equals( typ)) liste.add( auto);
    return liste;
  }

  /**
  * Sucht ein Auto vor Ort mit gegebenen Kennzeichen.
  * Falls es nicht vorhanden ist, wird null zurueckgegeben.
  * @param kennZeichen Kennzeichen des gesuchten Autos
  * @return Auto mit Kennzeichen vor Ort,
  *         null, falls nicht vorhanden
  */
  public Auto getAuto( String kennZeichen )
  {
    for( Auto auto: alleAutos)
      if( auto.getKennZeichen().equals( kennZeichen))
        return auto;
    return null;
  }
  
  /**
  * Ein Auto verlaesst den Ort.
  * @param auto Auto, welches den Ort verlassen soll
  * @return true, falls Auto den Ort verlassen hat
  */
  public boolean abmelden( Auto auto )
  {
    return alleAutos.remove( auto);
  }
  
  /**
  * Ein Auto kommt am Ort an.
  * @param auto Auto, welches am Ort ankommen soll
  * @return true, falls Auto am Ort angekommen ist
  */
  public boolean anmelden( Auto auto )
  {
    return alleAutos.add( auto);
  }

  /**
  * Gibt als String die Beschreibungen aller Autos vor Ort zurueck.
  * Falls kein Auto vorhanden ist, so wird das mitgeteilt.
  * @return Beschreibung aller Autos vor Ort oder
  *         "Es befinden sich keine Autos vor Ort."
  */
  public String toString()
  {
    String str = "";
    if( alleAutos.isEmpty())
      str = "Es befinden sich keine Autos vor Ort.";
    else
      for( Auto auto: alleAutos) str += "\n" + auto + "\n";
    return str;
  }
  
/* ----------------------------------------------------- */

  /**
  * Testprogramm:
  * Auf 3 Orte (Parkplatz, Strasse, Werkstatt) werden
  * 6 Testautos verteilt und die Verteilung angezeigt.
  *
  * Danach wird je ein Auto von der Strasse und ein Auto
  * aus der Werkstatt auf den Parkplatz verschoben.
  * Ein Auto wird verschrottet.
  * Die veraenderte Verteilung wird angezeigt.
  */
  public static void main( String[] args )
  {
// Orte
    Parkplatz parkplatz = new Parkplatz();
    Werkstatt werkstatt = new Werkstatt();
    Strasse strasse = new Strasse();
    
// Autos
    Ausstattung ausstattung;
    Auto auto;
    
// Kleinwagen -> Parkplatz
    ausstattung
    = new Ausstattung( true, true, true, false, false);
    ausstattung.setSonst( "neu");
    auto =  new Kleinwagen( ausstattung, 50);
    parkplatz.anmelden( auto);

    ausstattung
    = new Ausstattung( false, true, false, false, false);
    ausstattung.setSonst( "Multimedia"); 
    auto =  new Kleinwagen( ausstattung, 40);
    parkplatz.anmelden( auto);

// Mittelklassewagen -> Strasse
    ausstattung
    = new Ausstattung( false, true, true, true, false);
    auto = new Mittelklassewagen( ausstattung, 60);
    strasse.anmelden( auto);

    ausstattung
    = new Ausstattung( true, true, true, false, false);
    auto = new Mittelklassewagen( ausstattung, 60);
    strasse.anmelden( auto);

// Oberklassewagen -> Werkstatt
    ausstattung
    = new Ausstattung( false, true, true, false, false);
    ausstattung.setSonst( "Cabrio"); 
    auto = new Oberklassewagen( ausstattung, 90);
    werkstatt.anmelden( auto);

    ausstattung
    = new Ausstattung( true, true, true, true, false);
    auto = new Oberklassewagen( ausstattung, 80);
    werkstatt.anmelden( auto);

// Autos vor Ort
    System.out.println( "\nParkplatz:\n" + parkplatz);
    System.out.println( "\nWerkstatt:\n" + werkstatt);
    System.out.println( "\nStrasse:\n" + strasse);
    

// L-RA-1002: Strasse -> Parkplatz
    auto = strasse.getAuto( "L-RA-1002");
    if( auto != null)
    {
      strasse.abmelden( auto);
      parkplatz.anmelden( auto);
    }
    else
      System.out.println
      ( "\nL-RA-1002: befindet sich nicht auf der Strasse\n");

// L-RA-1004: Werkstatt -> Parkplatz
    auto = werkstatt.getAuto( "L-RA-1004");
    if( auto != null)
    {
      werkstatt.abmelden( auto);
      parkplatz.anmelden( auto);
    }
    else
      System.out.println
      ( "\nL-RA-1004: befindet sich nicht in der Werkstatt\n");

// Kleinwagen auf dem Parkplatz
/*
    LinkedList<Auto> autos
    = parkplatz.getAlleAutos( "Kleinwagen");
    System.out.println
    ( "\nAlle Kleinwagen auf dem Parkplatz:");
    for( Auto klein: autos)
       System.out.println( "\n" + klein);
*/

// L-RA-1005: Verschrotten    
    auto = werkstatt.getAuto( "L-RA-1005");
    if( auto != null)
      werkstatt.verschrotten( auto);
    else
      System.out.println
      ( "\nL-RA-1005: befindet sich nicht in der Werkstatt\n");
    System.out.println( "\nWerkstatt:\n" + werkstatt);
  
// Autos vor Ort
    System.out.println( "\nParkplatz:\n" + parkplatz);
    System.out.println( "\nWerkstatt:\n" + werkstatt);
    System.out.println( "\nStrasse:\n" + strasse);
  }
}